@@ -104,20 +104,6 @@ jomo1rancon<- function(Y, X=NULL, Z=NULL, clus, beta.start=NULL, u.start=NULL, l
     Yimp=Yimp2
     if (output==1) cat("Imputation number ", i, "registered", "\n")
   }
-  betapostmean<-apply(betapost, c(1,2), mean)
-  upostmean<-apply(upostall, c(1,2), mean)
-  omegapostmean<-apply(omegapost, c(1,2), mean)
-  covupostmean<-apply(covupost, c(1,2), mean)
-  if (output==1) {
-    cat("The posterior mean of the fixed effects estimates is:\n")
-    print(betapostmean)
-    cat("The posterior mean of the random effects estimates is:\n")
-    print(upostmean)
-    cat("The posterior mean of the level 1 covariance matrices is:\n")
-    print(omegapostmean)
-    cat("The posterior mean of the level 2 covariance matrix is:\n")
-    print(covupostmean)
-  }
   imp<-data.frame(imp)
   imp[,(ncol(Y)+ncol(X)+ncol(Z)+1)]<-factor(imp[,(ncol(Y)+ncol(X)+ncol(Z)+1)])
   levels(imp[,(ncol(Y)+ncol(X)+ncol(Z)+1)])<-previous_levels_clus
@@ -130,5 +116,28 @@ jomo1rancon<- function(Y, X=NULL, Z=NULL, clus, beta.start=NULL, u.start=NULL, l
   if (is.null(colnamz)) colnamz=paste("Z", 1:ncol(Z), sep = "")
   if (is.null(colnamx)) colnamx=paste("X", 1:ncol(X), sep = "")
   colnames(imp)<-c(colnamy,colnamx,colnamz,"clus","id","Imputation")
+  dimnames(betapost)[1] <- list(colnamx)
+  dimnames(betapost)[2] <- list(colnamy)
+  dimnames(omegapost)[1] <- list(colnamy)
+  dimnames(omegapost)[2] <- list(colnamy)
+  colnamcovu<-paste(colnamy,rep(colnamz,each=ncol(omegapost)),sep="*")
+  dimnames(covupost)[1] <- list(colnamcovu)
+  dimnames(covupost)[2] <- list(colnamcovu)
+  dimnames(upostall)[1]<-list(levels(clus))
+  dimnames(upostall)[2]<-list(colnamcovu)
+  betapostmean<-data.frame(apply(betapost, c(1,2), mean))
+  upostmean<-data.frame(apply(upostall, c(1,2), mean))
+  omegapostmean<-data.frame(apply(omegapost, c(1,2), mean))
+  covupostmean<-data.frame(apply(covupost, c(1,2), mean))
+  if (output==1) {
+    cat("The posterior mean of the fixed effects estimates is:\n")
+    print(t(betapostmean))
+    cat("\nThe posterior mean of the random effects estimates is:\n")
+    print(upostmean)
+    cat("\nThe posterior mean of the level 1 covariance matrices is:\n")
+    print(omegapostmean)
+    cat("\nThe posterior mean of the level 2 covariance matrix is:\n")
+    print(covupostmean)
+  }
   return(imp)
 }
\ No newline at end of file
