@@ -1023,8 +1023,9 @@ for (j=0; j<IY; j++) {
 		}
 	
 	}
-	if ((i+1)%fl==0) Rprintf("Iteration %d completed\n",i+1);
+	if ((i+1)%fl==0) Rprintf(".");
 }
+if (fl==1) Rprintf("\n");
 
 for(i=0;i<IY;i++)  {
 	for(j=0;j<JY;j++)  {
